@@ -31,8 +31,8 @@
 #ifndef OPM_ECL_MATERIAL_LAW_MANAGER_HPP
 #define OPM_ECL_MATERIAL_LAW_MANAGER_HPP
 
+#include <opm/material/fluidmatrixinteractions/SatCurveMultiplexer.hpp>
 #include <opm/material/fluidmatrixinteractions/EclTwoPhaseMaterialParams.hpp>
-#include <opm/material/fluidmatrixinteractions/PiecewiseLinearTwoPhaseMaterial.hpp>
 #include <opm/material/fluidmatrixinteractions/EclEpsTwoPhaseLaw.hpp>
 #include <opm/material/fluidmatrixinteractions/EclHysteresisTwoPhaseLaw.hpp>
 #include <opm/material/fluidmatrixinteractions/EclEpsScalingPoints.hpp>
@@ -49,6 +49,7 @@
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableColumn.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FaceDir.hpp>
 
 #include <algorithm>
 #include <cassert>
@@ -68,61 +69,61 @@ template <class TraitsT>
 class EclMaterialLawManager
 {
 private:
-    typedef TraitsT Traits;
-    typedef typename Traits::Scalar Scalar;
+    using Traits = TraitsT;
+    using Scalar = typename Traits::Scalar;
     enum { waterPhaseIdx = Traits::wettingPhaseIdx };
     enum { oilPhaseIdx = Traits::nonWettingPhaseIdx };
     enum { gasPhaseIdx = Traits::gasPhaseIdx };
     enum { numPhases = Traits::numPhases };
 
-    typedef TwoPhaseMaterialTraits<Scalar, oilPhaseIdx, gasPhaseIdx> GasOilTraits;
-    typedef TwoPhaseMaterialTraits<Scalar, waterPhaseIdx, oilPhaseIdx> OilWaterTraits;
-    typedef TwoPhaseMaterialTraits<Scalar, waterPhaseIdx, gasPhaseIdx> GasWaterTraits;
+    using GasOilTraits = TwoPhaseMaterialTraits<Scalar, oilPhaseIdx, gasPhaseIdx>;
+    using OilWaterTraits = TwoPhaseMaterialTraits<Scalar, waterPhaseIdx, oilPhaseIdx>;
+    using GasWaterTraits = TwoPhaseMaterialTraits<Scalar, waterPhaseIdx, gasPhaseIdx>;
 
     // the two-phase material law which is defined on effective (unscaled) saturations
-    typedef PiecewiseLinearTwoPhaseMaterial<GasOilTraits> GasOilEffectiveTwoPhaseLaw;
-    typedef PiecewiseLinearTwoPhaseMaterial<OilWaterTraits> OilWaterEffectiveTwoPhaseLaw;
-    typedef PiecewiseLinearTwoPhaseMaterial<GasWaterTraits> GasWaterEffectiveTwoPhaseLaw;
+    using GasOilEffectiveTwoPhaseLaw = SatCurveMultiplexer<GasOilTraits>;
+    using OilWaterEffectiveTwoPhaseLaw = SatCurveMultiplexer<OilWaterTraits>;
+    using GasWaterEffectiveTwoPhaseLaw = SatCurveMultiplexer<GasWaterTraits>;
 
-    typedef typename GasOilEffectiveTwoPhaseLaw::Params GasOilEffectiveTwoPhaseParams;
-    typedef typename OilWaterEffectiveTwoPhaseLaw::Params OilWaterEffectiveTwoPhaseParams;
-    typedef typename GasWaterEffectiveTwoPhaseLaw::Params GasWaterEffectiveTwoPhaseParams;
+    using GasOilEffectiveTwoPhaseParams = typename GasOilEffectiveTwoPhaseLaw::Params;
+    using OilWaterEffectiveTwoPhaseParams = typename OilWaterEffectiveTwoPhaseLaw::Params;
+    using GasWaterEffectiveTwoPhaseParams = typename GasWaterEffectiveTwoPhaseLaw::Params;
 
     // the two-phase material law which is defined on absolute (scaled) saturations
-    typedef EclEpsTwoPhaseLaw<GasOilEffectiveTwoPhaseLaw> GasOilEpsTwoPhaseLaw;
-    typedef EclEpsTwoPhaseLaw<OilWaterEffectiveTwoPhaseLaw> OilWaterEpsTwoPhaseLaw;
-     typedef EclEpsTwoPhaseLaw<GasWaterEffectiveTwoPhaseLaw> GasWaterEpsTwoPhaseLaw;
-    typedef typename GasOilEpsTwoPhaseLaw::Params GasOilEpsTwoPhaseParams;
-    typedef typename OilWaterEpsTwoPhaseLaw::Params OilWaterEpsTwoPhaseParams;
-    typedef typename GasWaterEpsTwoPhaseLaw::Params GasWaterEpsTwoPhaseParams;
+    using GasOilEpsTwoPhaseLaw = EclEpsTwoPhaseLaw<GasOilEffectiveTwoPhaseLaw>;
+    using OilWaterEpsTwoPhaseLaw = EclEpsTwoPhaseLaw<OilWaterEffectiveTwoPhaseLaw>;
+    using GasWaterEpsTwoPhaseLaw = EclEpsTwoPhaseLaw<GasWaterEffectiveTwoPhaseLaw>;
+    using GasOilEpsTwoPhaseParams = typename GasOilEpsTwoPhaseLaw::Params;
+    using OilWaterEpsTwoPhaseParams = typename OilWaterEpsTwoPhaseLaw::Params;
+    using GasWaterEpsTwoPhaseParams = typename GasWaterEpsTwoPhaseLaw::Params;
 
     // the scaled two-phase material laws with hystersis
-    typedef EclHysteresisTwoPhaseLaw<GasOilEpsTwoPhaseLaw> GasOilTwoPhaseLaw;
-    typedef EclHysteresisTwoPhaseLaw<OilWaterEpsTwoPhaseLaw> OilWaterTwoPhaseLaw;
-    typedef EclHysteresisTwoPhaseLaw<GasWaterEpsTwoPhaseLaw> GasWaterTwoPhaseLaw;
-    typedef typename GasOilTwoPhaseLaw::Params GasOilTwoPhaseHystParams;
-    typedef typename OilWaterTwoPhaseLaw::Params OilWaterTwoPhaseHystParams;
-    typedef typename GasWaterTwoPhaseLaw::Params GasWaterTwoPhaseHystParams;
+    using GasOilTwoPhaseLaw = EclHysteresisTwoPhaseLaw<GasOilEpsTwoPhaseLaw>;
+    using OilWaterTwoPhaseLaw = EclHysteresisTwoPhaseLaw<OilWaterEpsTwoPhaseLaw>;
+    using GasWaterTwoPhaseLaw = EclHysteresisTwoPhaseLaw<GasWaterEpsTwoPhaseLaw>;
+    using GasOilTwoPhaseHystParams = typename GasOilTwoPhaseLaw::Params;
+    using OilWaterTwoPhaseHystParams = typename OilWaterTwoPhaseLaw::Params;
+    using GasWaterTwoPhaseHystParams = typename GasWaterTwoPhaseLaw::Params;
 
 public:
     // the three-phase material law used by the simulation
-    typedef EclMultiplexerMaterial<Traits, GasOilTwoPhaseLaw, OilWaterTwoPhaseLaw, GasWaterTwoPhaseLaw> MaterialLaw;
-    typedef typename MaterialLaw::Params MaterialLawParams;
+    using MaterialLaw = EclMultiplexerMaterial<Traits, GasOilTwoPhaseLaw, OilWaterTwoPhaseLaw, GasWaterTwoPhaseLaw>;
+    using MaterialLawParams = typename MaterialLaw::Params;
 
 private:
     // internal typedefs
-    typedef std::vector<std::shared_ptr<GasOilEffectiveTwoPhaseParams> > GasOilEffectiveParamVector;
-    typedef std::vector<std::shared_ptr<OilWaterEffectiveTwoPhaseParams> > OilWaterEffectiveParamVector;
-    typedef std::vector<std::shared_ptr<GasWaterEffectiveTwoPhaseParams> > GasWaterEffectiveParamVector;
+    using GasOilEffectiveParamVector = std::vector<std::shared_ptr<GasOilEffectiveTwoPhaseParams>>;
+    using OilWaterEffectiveParamVector = std::vector<std::shared_ptr<OilWaterEffectiveTwoPhaseParams>>;
+    using GasWaterEffectiveParamVector = std::vector<std::shared_ptr<GasWaterEffectiveTwoPhaseParams>>;
 
-    typedef std::vector<std::shared_ptr<EclEpsScalingPoints<Scalar> > > GasOilScalingPointsVector;
-    typedef std::vector<std::shared_ptr<EclEpsScalingPoints<Scalar> > > OilWaterScalingPointsVector;
-    typedef std::vector<std::shared_ptr<EclEpsScalingPoints<Scalar> > > GasWaterScalingPointsVector;
-    typedef std::vector<EclEpsScalingPointsInfo<Scalar>> OilWaterScalingInfoVector;
-    typedef std::vector<std::shared_ptr<GasOilTwoPhaseHystParams> > GasOilParamVector;
-    typedef std::vector<std::shared_ptr<OilWaterTwoPhaseHystParams> > OilWaterParamVector;
-    typedef std::vector<std::shared_ptr<GasWaterTwoPhaseHystParams> > GasWaterParamVector;
-    typedef std::vector<std::shared_ptr<MaterialLawParams> > MaterialLawParamsVector;
+    using GasOilScalingPointsVector = std::vector<std::shared_ptr<EclEpsScalingPoints<Scalar>>>;
+    using OilWaterScalingPointsVector = std::vector<std::shared_ptr<EclEpsScalingPoints<Scalar>>>;
+    using GasWaterScalingPointsVector = std::vector<std::shared_ptr<EclEpsScalingPoints<Scalar>>>;
+    using OilWaterScalingInfoVector = std::vector<EclEpsScalingPointsInfo<Scalar>>;
+    using GasOilParamVector = std::vector<std::shared_ptr<GasOilTwoPhaseHystParams>>;
+    using OilWaterParamVector = std::vector<std::shared_ptr<OilWaterTwoPhaseHystParams>>;
+    using GasWaterParamVector = std::vector<std::shared_ptr<GasWaterTwoPhaseHystParams>>;
+    using MaterialLawParamsVector = std::vector<std::shared_ptr<MaterialLawParams>>;
 
 public:
     EclMaterialLawManager()
@@ -221,8 +222,21 @@ public:
                 satnumRegionArray_[elemIdx] = satnumRawData[elemIdx] - 1;
             }
         }
-        else
+        else {
             std::fill(satnumRegionArray_.begin(), satnumRegionArray_.end(), 0);
+        }
+        auto copy_krnum = [&eclState, numCompressedElems](std::vector<int>& dest, const std::string keyword) {
+            if (eclState.fieldProps().has_int(keyword)) {
+                dest.resize(numCompressedElems);
+                const auto& satnumRawData = eclState.fieldProps().get_int(keyword);
+                for (unsigned elemIdx = 0; elemIdx < numCompressedElems; ++elemIdx) {
+                    dest[elemIdx] = satnumRawData[elemIdx] - 1;
+                }
+            }
+        };
+        copy_krnum(krnumXArray_, "KRNUMX");
+        copy_krnum(krnumYArray_, "KRNUMY");
+        copy_krnum(krnumZArray_, "KRNUMZ");
 
         // create the information for the imbibition region (IMBNUM). By default this is
         // the same as the saturation region (SATNUM)
@@ -289,7 +303,9 @@ public:
                 gasOilDrainParams.setEffectiveLawParams(gasOilEffectiveParamVector_[satRegionIdx]);
                 gasOilDrainParams.finalize();
 
-                gasOilParams->setDrainageParams(gasOilDrainParams);
+                gasOilParams->setDrainageParams(gasOilDrainParams,
+                                                gasOilScaledInfo,
+                                                EclGasOilSystem);
             }
 
             if (hasOil && hasWater) {
@@ -300,7 +316,9 @@ public:
                 oilWaterDrainParams.setEffectiveLawParams(oilWaterEffectiveParamVector_[satRegionIdx]);
                 oilWaterDrainParams.finalize();
 
-                oilWaterParams->setDrainageParams(oilWaterDrainParams);
+                oilWaterParams->setDrainageParams(oilWaterDrainParams,
+                                                  owinfo,
+                                                  EclOilWaterSystem);
             }
 
             if (hasGas && hasWater && !hasOil) {
@@ -311,7 +329,9 @@ public:
                 gasWaterDrainParams.setEffectiveLawParams(gasWaterEffectiveParamVector_[satRegionIdx]);
                 gasWaterDrainParams.finalize();
 
-                gasWaterParams->setDrainageParams(gasWaterDrainParams);
+                gasWaterParams->setDrainageParams(gasWaterDrainParams,
+                                                  gasWaterScaledInfo,
+                                                  EclGasWaterSystem);
             }
 
             if (enableHysteresis()) {
@@ -359,7 +379,7 @@ public:
                     oilWaterImbParamsHyst.finalize();
 
                     oilWaterParams->setImbibitionParams(oilWaterImbParamsHyst,
-                                                        gasOilScaledImbInfo,
+                                                        oilWaterScaledImbInfo,
                                                         EclOilWaterSystem);
                 }
 
@@ -423,27 +443,27 @@ public:
                 Sw = elemScaledEpsInfo.Swl;
 
             // specify a fluid state which only stores the saturations
-            typedef SimpleModularFluidState<Scalar,
-                                            numPhases,
-                                            /*numComponents=*/0,
-                                            /*FluidSystem=*/void, /* -> don't care */
-                                            /*storePressure=*/false,
-                                            /*storeTemperature=*/false,
-                                            /*storeComposition=*/false,
-                                            /*storeFugacity=*/false,
-                                            /*storeSaturation=*/true,
-                                            /*storeDensity=*/false,
-                                            /*storeViscosity=*/false,
-                                            /*storeEnthalpy=*/false> FluidState;
+            using FluidState = SimpleModularFluidState<Scalar,
+                                                       numPhases,
+                                                       /*numComponents=*/0,
+                                                       /*FluidSystem=*/void, /* -> don't care */
+                                                       /*storePressure=*/false,
+                                                       /*storeTemperature=*/false,
+                                                       /*storeComposition=*/false,
+                                                       /*storeFugacity=*/false,
+                                                       /*storeSaturation=*/true,
+                                                       /*storeDensity=*/false,
+                                                       /*storeViscosity=*/false,
+                                                       /*storeEnthalpy=*/false>;
             FluidState fs;
             fs.setSaturation(waterPhaseIdx, Sw);
             fs.setSaturation(gasPhaseIdx, 0);
             fs.setSaturation(oilPhaseIdx, 0);
-            Scalar pc[numPhases] = { 0 };
+            std::array<Scalar, numPhases> pc = { 0 };
             MaterialLaw::capillaryPressures(pc, materialLawParams(elemIdx), fs);
 
             Scalar pcowAtSw = pc[oilPhaseIdx] - pc[waterPhaseIdx];
-            const Scalar pcowAtSwThreshold = 1.0; //Pascal
+            constexpr const Scalar pcowAtSwThreshold = 1.0; //Pascal
             // avoid divison by very small number
             if (std::abs(pcowAtSw) > pcowAtSwThreshold) {
                 elemScaledEpsInfo.maxPcow *= pcow/pcowAtSw;
@@ -565,6 +585,35 @@ public:
     int satnumRegionIdx(unsigned elemIdx) const
     { return satnumRegionArray_[elemIdx]; }
 
+    int getKrnumSatIdx(unsigned elemIdx, FaceDir::DirEnum facedir) const {
+        using Dir = FaceDir::DirEnum;
+        const std::vector<int>* array = nullptr;
+        switch(facedir) {
+            case Dir::XPlus:
+                array = &krnumXArray_;
+                break;
+            case Dir::YPlus:
+                array = &krnumYArray_;
+                break;
+            case Dir::ZPlus:
+                array = &krnumZArray_;
+                break;
+            default:
+                throw std::runtime_error("Unknown face direction");
+        }
+        if (array->size() > 0) {
+            return (*array)[elemIdx];
+        }
+        else {
+            return satnumRegionArray_[elemIdx];
+        }
+    }
+    bool hasDirectionalRelperms() const {
+        if (krnumXArray_.size() > 0 || krnumYArray_.size() > 0 || krnumZArray_.size() > 0) {
+            return true;
+        }
+        return false;
+    }
     int imbnumRegionIdx(unsigned elemIdx) const
     { return imbnumRegionArray_[elemIdx]; }
 
@@ -741,6 +790,45 @@ private:
             else if (!slgofTables.empty())
                 readGasOilEffectiveParametersSlgof_(effParams, Swco, tolcrit,
                                                     slgofTables.getTable<SlgofTable>(satRegionIdx));
+            else if ( !tableManager.getSgofletTable().empty() ) {
+                const auto& letSgofTab = tableManager.getSgofletTable()[satRegionIdx];
+                const std::vector<Scalar> dum; // dummy arg to comform with existing interface
+
+                effParams.setApproach(SatCurveMultiplexerApproach::LETApproach);
+                auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::LETApproach>();
+
+                // S=(So-Sogcr)/(1-Sogcr-Sgcr-Swco),  krog = Krt*S^L/[S^L+E*(1.0-S)^T]
+                const Scalar s_min_w = letSgofTab.s2_critical;
+                const Scalar s_max_w = 1.0-letSgofTab.s1_critical-Swco;
+                const std::vector<Scalar>& letCoeffsOil = {s_min_w, s_max_w,
+                                                           static_cast<Scalar>(letSgofTab.l2_relperm),
+                                                           static_cast<Scalar>(letSgofTab.e2_relperm),
+                                                           static_cast<Scalar>(letSgofTab.t2_relperm),
+                                                           static_cast<Scalar>(letSgofTab.krt2_relperm)};
+                realParams.setKrwSamples(letCoeffsOil, dum);
+
+                // S=(1-So-Sgcr-Swco)/(1-Sogcr-Sgcr-Swco), krg = Krt*S^L/[S^L+E*(1.0-S)^T]
+                const Scalar s_min_nw = letSgofTab.s1_critical+Swco;
+                const Scalar s_max_nw = 1.0-letSgofTab.s2_critical;
+                const std::vector<Scalar>& letCoeffsGas = {s_min_nw, s_max_nw,
+                                                           static_cast<Scalar>(letSgofTab.l1_relperm),
+                                                           static_cast<Scalar>(letSgofTab.e1_relperm),
+                                                           static_cast<Scalar>(letSgofTab.t1_relperm),
+                                                           static_cast<Scalar>(letSgofTab.krt1_relperm)};
+                realParams.setKrnSamples(letCoeffsGas, dum);
+
+                // S=(So-Sorg)/(1-Sorg-Sgl-Swco), Pc = Pct + (pcir_pc-Pct)*(1-S)^L/[(1-S)^L+E*S^T]
+                const std::vector<Scalar>& letCoeffsPc = {static_cast<Scalar>(letSgofTab.s2_residual),
+                                                          static_cast<Scalar>(letSgofTab.s1_residual+Swco),
+                                                          static_cast<Scalar>(letSgofTab.l_pc),
+                                                          static_cast<Scalar>(letSgofTab.e_pc),
+                                                          static_cast<Scalar>(letSgofTab.t_pc),
+                                                          static_cast<Scalar>(letSgofTab.pcir_pc),
+                                                          static_cast<Scalar>(letSgofTab.pct_pc)};
+                realParams.setPcnwSamples(letCoeffsPc, dum);
+
+                realParams.finalize();
+            }
             break;
         }
 
@@ -775,10 +863,13 @@ private:
             SoSamples[sampleIdx] = (1.0 - Swco) - sgofTable.get("SG", sampleIdx);
         }
 
-        effParams.setKrwSamples(SoSamples, normalizeKrValues_(tolcrit, sgofTable.getColumn("KROG")));
-        effParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, sgofTable.getColumn("KRG")));
-        effParams.setPcnwSamples(SoSamples, sgofTable.getColumn("PCOG").vectorCopy());
-        effParams.finalize();
+        effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+        auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
+        realParams.setKrwSamples(SoSamples, normalizeKrValues_(tolcrit, sgofTable.getColumn("KROG")));
+        realParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, sgofTable.getColumn("KRG")));
+        realParams.setPcnwSamples(SoSamples, sgofTable.getColumn("PCOG").vectorCopy());
+        realParams.finalize();
     }
 
     void readGasOilEffectiveParametersSlgof_(GasOilEffectiveTwoPhaseParams& effParams,
@@ -792,10 +883,13 @@ private:
             SoSamples[sampleIdx] = slgofTable.get("SL", sampleIdx) - Swco;
         }
 
-        effParams.setKrwSamples(SoSamples, normalizeKrValues_(tolcrit, slgofTable.getColumn("KROG")));
-        effParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, slgofTable.getColumn("KRG")));
-        effParams.setPcnwSamples(SoSamples, slgofTable.getColumn("PCOG").vectorCopy());
-        effParams.finalize();
+        effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+        auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
+        realParams.setKrwSamples(SoSamples, normalizeKrValues_(tolcrit, slgofTable.getColumn("KROG")));
+        realParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, slgofTable.getColumn("KRG")));
+        realParams.setPcnwSamples(SoSamples, slgofTable.getColumn("PCOG").vectorCopy());
+        realParams.finalize();
     }
 
     void readGasOilEffectiveParametersFamily2_(GasOilEffectiveTwoPhaseParams& effParams,
@@ -811,10 +905,13 @@ private:
             SoSamples[sampleIdx] = (1.0 - Swco) - sgfnTable.get("SG", sampleIdx);
         }
 
-        effParams.setKrwSamples(SoColumn, normalizeKrValues_(tolcrit, sof3Table.getColumn("KROG")));
-        effParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, sgfnTable.getColumn("KRG")));
-        effParams.setPcnwSamples(SoSamples, sgfnTable.getColumn("PCOG").vectorCopy());
-        effParams.finalize();
+        effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+        auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
+        realParams.setKrwSamples(SoColumn, normalizeKrValues_(tolcrit, sof3Table.getColumn("KROG")));
+        realParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, sgfnTable.getColumn("KRG")));
+        realParams.setPcnwSamples(SoSamples, sgfnTable.getColumn("PCOG").vectorCopy());
+        realParams.finalize();
     }
 
     void readGasOilEffectiveParametersFamily2_(GasOilEffectiveTwoPhaseParams& effParams,
@@ -830,10 +927,13 @@ private:
             SoSamples[sampleIdx] = (1.0 - Swco) - sgfnTable.get("SG", sampleIdx);
         }
 
-        effParams.setKrwSamples(SoColumn, normalizeKrValues_(tolcrit, sof2Table.getColumn("KRO")));
-        effParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, sgfnTable.getColumn("KRG")));
-        effParams.setPcnwSamples(SoSamples, sgfnTable.getColumn("PCOG").vectorCopy());
-        effParams.finalize();
+        effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+        auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
+        realParams.setKrwSamples(SoColumn, normalizeKrValues_(tolcrit, sof2Table.getColumn("KRO")));
+        realParams.setKrnSamples(SoSamples, normalizeKrValues_(tolcrit, sgfnTable.getColumn("KRG")));
+        realParams.setPcnwSamples(SoSamples, sgfnTable.getColumn("PCOG").vectorCopy());
+        realParams.finalize();
     }
 
     template <class Container>
@@ -856,13 +956,57 @@ private:
         switch (eclState.runspec().saturationFunctionControls().family()) {
         case SatFuncControls::KeywordFamily::Family_I:
         {
-            const auto& swofTable = tableManager.getSwofTables().getTable<SwofTable>(satRegionIdx);
-            const std::vector<double> SwColumn = swofTable.getColumn("SW").vectorCopy();
+            if (tableManager.hasTables("SWOF")) {
+                const auto& swofTable = tableManager.getSwofTables().getTable<SwofTable>(satRegionIdx);
+                const std::vector<double> SwColumn = swofTable.getColumn("SW").vectorCopy();
 
-            effParams.setKrwSamples(SwColumn, normalizeKrValues_(tolcrit, swofTable.getColumn("KRW")));
-            effParams.setKrnSamples(SwColumn, normalizeKrValues_(tolcrit, swofTable.getColumn("KROW")));
-            effParams.setPcnwSamples(SwColumn, swofTable.getColumn("PCOW").vectorCopy());
-            effParams.finalize();
+                effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+                auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
+                realParams.setKrwSamples(SwColumn, normalizeKrValues_(tolcrit, swofTable.getColumn("KRW")));
+                realParams.setKrnSamples(SwColumn, normalizeKrValues_(tolcrit, swofTable.getColumn("KROW")));
+                realParams.setPcnwSamples(SwColumn, swofTable.getColumn("PCOW").vectorCopy());
+                realParams.finalize();
+            }
+            else if ( !tableManager.getSwofletTable().empty() ) {
+                const auto& letTab = tableManager.getSwofletTable()[satRegionIdx];
+                const std::vector<Scalar> dum; // dummy arg to conform with existing interface
+
+                effParams.setApproach(SatCurveMultiplexerApproach::LETApproach);
+                auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::LETApproach>();
+
+                // S=(Sw-Swcr)/(1-Sowcr-Swcr),  krw = Krt*S^L/[S^L+E*(1.0-S)^T]
+                const Scalar s_min_w = letTab.s1_critical;
+                const Scalar s_max_w = 1.0-letTab.s2_critical;
+                const std::vector<Scalar>& letCoeffsWat = {s_min_w, s_max_w,
+                                                           static_cast<Scalar>(letTab.l1_relperm),
+                                                           static_cast<Scalar>(letTab.e1_relperm),
+                                                           static_cast<Scalar>(letTab.t1_relperm),
+                                                           static_cast<Scalar>(letTab.krt1_relperm)};
+                realParams.setKrwSamples(letCoeffsWat, dum);
+
+                // S=(So-Sowcr)/(1-Sowcr-Swcr), krow = Krt*S^L/[S^L+E*(1.0-S)^T]
+                const Scalar s_min_nw = letTab.s2_critical;
+                const Scalar s_max_nw = 1.0-letTab.s1_critical;
+                const std::vector<Scalar>& letCoeffsOil = {s_min_nw, s_max_nw,
+                                                           static_cast<Scalar>(letTab.l2_relperm),
+                                                           static_cast<Scalar>(letTab.e2_relperm),
+                                                           static_cast<Scalar>(letTab.t2_relperm),
+                                                           static_cast<Scalar>(letTab.krt2_relperm)};
+                realParams.setKrnSamples(letCoeffsOil, dum);
+
+                // S=(Sw-Swco)/(1-Swco-Sorw), Pc = Pct + (Pcir-Pct)*(1-S)^L/[(1-S)^L+E*S^T]
+                const std::vector<Scalar>& letCoeffsPc = {static_cast<Scalar>(letTab.s1_residual),
+                                                          static_cast<Scalar>(letTab.s2_residual),
+                                                          static_cast<Scalar>(letTab.l_pc),
+                                                          static_cast<Scalar>(letTab.e_pc),
+                                                          static_cast<Scalar>(letTab.t_pc),
+                                                          static_cast<Scalar>(letTab.pcir_pc),
+                                                          static_cast<Scalar>(letTab.pct_pc)};
+                realParams.setPcnwSamples(letCoeffsPc, dum);
+
+                realParams.finalize();
+            }
             break;
         }
 
@@ -870,8 +1014,12 @@ private:
         {
             const auto& swfnTable = tableManager.getSwfnTables().getTable<SwfnTable>(satRegionIdx);
             const std::vector<double> SwColumn = swfnTable.getColumn("SW").vectorCopy();
-            effParams.setKrwSamples(SwColumn, normalizeKrValues_(tolcrit, swfnTable.getColumn("KRW")));
-            effParams.setPcnwSamples(SwColumn, swfnTable.getColumn("PCOW").vectorCopy());
+
+            effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+            auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
+            realParams.setKrwSamples(SwColumn, normalizeKrValues_(tolcrit, swfnTable.getColumn("KRW")));
+            realParams.setPcnwSamples(SwColumn, swfnTable.getColumn("PCOW").vectorCopy());
 
             if (!hasGas) {
                 const auto& sof2Table = tableManager.getSof2Tables().getTable<Sof2Table>(satRegionIdx);
@@ -880,7 +1028,7 @@ private:
                 for (size_t sampleIdx = 0; sampleIdx < sof2Table.numRows(); ++ sampleIdx)
                     SwSamples[sampleIdx] = 1 - sof2Table.get("SO", sampleIdx);
 
-                effParams.setKrnSamples(SwSamples, normalizeKrValues_(tolcrit, sof2Table.getColumn("KRO")));
+                realParams.setKrnSamples(SwSamples, normalizeKrValues_(tolcrit, sof2Table.getColumn("KRO")));
             } else {
                 const auto& sof3Table = tableManager.getSof3Tables().getTable<Sof3Table>(satRegionIdx);
                 // convert the saturations of the SOF3 keyword from oil to water saturations
@@ -888,9 +1036,9 @@ private:
                 for (size_t sampleIdx = 0; sampleIdx < sof3Table.numRows(); ++ sampleIdx)
                     SwSamples[sampleIdx] = 1 - sof3Table.get("SO", sampleIdx);
 
-                effParams.setKrnSamples(SwSamples, normalizeKrValues_(tolcrit, sof3Table.getColumn("KROW")));
+                realParams.setKrnSamples(SwSamples, normalizeKrValues_(tolcrit, sof3Table.getColumn("KROW")));
             }
-            effParams.finalize();
+            realParams.finalize();
             break;
         }
 
@@ -929,17 +1077,20 @@ private:
             const SgfnTable& sgfnTable = tableManager.getSgfnTables().getTable<SgfnTable>( satRegionIdx );
             const SwfnTable& swfnTable = tableManager.getSwfnTables().getTable<SwfnTable>( satRegionIdx );
 
+            effParams.setApproach(SatCurveMultiplexerApproach::PiecewiseLinearApproach);
+            auto& realParams = effParams.template getRealParams<SatCurveMultiplexerApproach::PiecewiseLinearApproach>();
+
             std::vector<double> SwColumn = swfnTable.getColumn("SW").vectorCopy();
-            
-            effParams.setKrwSamples(SwColumn, normalizeKrValues_(tolcrit, swfnTable.getColumn("KRW")));
+
+            realParams.setKrwSamples(SwColumn, normalizeKrValues_(tolcrit, swfnTable.getColumn("KRW")));
             std::vector<double> SwSamples(sgfnTable.numRows());
             for (size_t sampleIdx = 0; sampleIdx < sgfnTable.numRows(); ++ sampleIdx)
                 SwSamples[sampleIdx] = 1 - sgfnTable.get("SG", sampleIdx);
-            effParams.setKrnSamples(SwSamples, normalizeKrValues_(tolcrit, sgfnTable.getColumn("KRG")));
+            realParams.setKrnSamples(SwSamples, normalizeKrValues_(tolcrit, sgfnTable.getColumn("KRG")));
             //Capillary pressure is read from SWFN. 
             //For gas-water system the capillary pressure column values are set to 0 in SGFN
-            effParams.setPcnwSamples(SwColumn, swfnTable.getColumn("PCOW").vectorCopy());
-            effParams.finalize();
+            realParams.setPcnwSamples(SwColumn, swfnTable.getColumn("PCOW").vectorCopy());
+            realParams.finalize();
                        
             break;
         }
@@ -1109,6 +1260,9 @@ private:
     std::vector<MaterialLawParams> materialLawParams_;
 
     std::vector<int> satnumRegionArray_;
+    std::vector<int> krnumXArray_;
+    std::vector<int> krnumYArray_;
+    std::vector<int> krnumZArray_;
     std::vector<int> imbnumRegionArray_;
     std::vector<Scalar> stoneEtas;
 
