@@ -139,18 +139,18 @@ auto getSaltSaturation_(typename std::enable_if<HasMember_saltSaturation<FluidSt
 template <class Scalar, class IndexTraits = BlackOilDefaultIndexTraits>
 class BlackOilFluidSystem : public BaseFluidSystem<Scalar, BlackOilFluidSystem<Scalar, IndexTraits> >
 {
-    typedef BlackOilFluidSystem ThisType;
+    using ThisType = BlackOilFluidSystem;
 
 public:
-    typedef GasPvtMultiplexer<Scalar> GasPvt;
-    typedef OilPvtMultiplexer<Scalar> OilPvt;
-    typedef WaterPvtMultiplexer<Scalar> WaterPvt;
+    using GasPvt = GasPvtMultiplexer<Scalar>;
+    using OilPvt = OilPvtMultiplexer<Scalar>;
+    using WaterPvt = WaterPvtMultiplexer<Scalar>;
 
     //! \copydoc BaseFluidSystem::ParameterCache
     template <class EvaluationT>
     struct ParameterCache : public NullParameterCache<EvaluationT>
     {
-        typedef EvaluationT Evaluation;
+        using Evaluation = EvaluationT;
 
     public:
         ParameterCache(Scalar maxOilSat = 1.0, unsigned regionIdx=0)
@@ -464,14 +464,14 @@ public:
      ****************************************/
 
     //! \copydoc BaseFluidSystem::numPhases
-    static const unsigned numPhases = 3;
+    static constexpr unsigned numPhases = 3;
 
     //! Index of the water phase
-    static const unsigned waterPhaseIdx = IndexTraits::waterPhaseIdx;
+    static constexpr unsigned waterPhaseIdx = IndexTraits::waterPhaseIdx;
     //! Index of the oil phase
-    static const unsigned oilPhaseIdx = IndexTraits::oilPhaseIdx;
+    static constexpr unsigned oilPhaseIdx = IndexTraits::oilPhaseIdx;
     //! Index of the gas phase
-    static const unsigned gasPhaseIdx = IndexTraits::gasPhaseIdx;
+    static constexpr unsigned gasPhaseIdx = IndexTraits::gasPhaseIdx;
 
     //! The pressure at the surface
     static Scalar surfacePressure;
@@ -507,14 +507,14 @@ public:
      ****************************************/
 
     //! \copydoc BaseFluidSystem::numComponents
-    static const unsigned numComponents = 3;
+    static constexpr unsigned numComponents = 3;
 
     //! Index of the oil component
-    static const unsigned oilCompIdx = IndexTraits::oilCompIdx;
+    static constexpr unsigned oilCompIdx = IndexTraits::oilCompIdx;
     //! Index of the water component
-    static const unsigned waterCompIdx = IndexTraits::waterCompIdx;
+    static constexpr unsigned waterCompIdx = IndexTraits::waterCompIdx;
     //! Index of the gas component
-    static const unsigned gasCompIdx = IndexTraits::gasCompIdx;
+    static constexpr unsigned gasCompIdx = IndexTraits::gasCompIdx;
 
 protected:
     static unsigned char numActivePhases_;
@@ -729,20 +729,32 @@ public:
         }
 
         case gasPhaseIdx: {
+             if (enableVaporizedOil() && enableVaporizedWater()) {
+                // gas containing vaporized oil and vaporized water
+                const LhsEval& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+
+                return
+                    bg*referenceDensity(gasPhaseIdx, regionIdx)
+                    + Rv*bg*referenceDensity(oilPhaseIdx, regionIdx)
+                    + Rvw*bg*referenceDensity(waterPhaseIdx, regionIdx);
+            }
             if (enableVaporizedOil()) {
                 // miscible gas
+                const LhsEval Rvw(0.0);
                 const LhsEval& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
 
                 return
                     bg*referenceDensity(gasPhaseIdx, regionIdx)
                     + Rv*bg*referenceDensity(oilPhaseIdx, regionIdx);
             }
-            //vaporized oil simultaneous with vaporized water is not yet supported
             if (enableVaporizedWater()) {
                 // gas containing vaporized water
+                const LhsEval Rv(0.0);
                 const LhsEval& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
-                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rvw);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
 
                 return
                     bg*referenceDensity(gasPhaseIdx, regionIdx)
@@ -751,7 +763,8 @@ public:
 
             // immiscible gas
             const LhsEval Rv(0.0);
-            const auto& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv);
+            const LhsEval Rvw(0.0);
+            const auto& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
             return bg*referenceDensity(phaseIdx, regionIdx);
         }
 
@@ -801,20 +814,34 @@ public:
         }
 
         case gasPhaseIdx: {
+            if (enableVaporizedOil() && enableVaporizedWater()) {
+                // gas containing vaporized oil and vaporized water
+                const LhsEval& Rv = saturatedDissolutionFactor<FluidState, LhsEval>(fluidState, gasPhaseIdx, regionIdx);
+                const LhsEval& Rvw = saturatedVaporizationFactor<FluidState, LhsEval>(fluidState, gasPhaseIdx, regionIdx);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+
+                return
+                    bg*referenceDensity(gasPhaseIdx, regionIdx)
+                    + Rv*bg*referenceDensity(oilPhaseIdx, regionIdx) 
+                    + Rvw*bg*referenceDensity(waterPhaseIdx, regionIdx) ;
+            }
+
             if (enableVaporizedOil()) {
                 // miscible gas
+                const LhsEval Rvw(0.0);
                 const LhsEval& Rv = saturatedDissolutionFactor<FluidState, LhsEval>(fluidState, gasPhaseIdx, regionIdx);
-                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
 
                 return
                     bg*referenceDensity(gasPhaseIdx, regionIdx)
                     + Rv*bg*referenceDensity(oilPhaseIdx, regionIdx);
             }
-            //vaporized oil simultaneous with vaporized water is not yet supported
+
             if (enableVaporizedWater()) {
                 // gas containing vaporized water
+                const LhsEval Rv(0.0);
                 const LhsEval& Rvw = saturatedVaporizationFactor<FluidState, LhsEval>(fluidState, gasPhaseIdx, regionIdx);
-                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rvw);
+                const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
 
                 return
                     bg*referenceDensity(gasPhaseIdx, regionIdx)
@@ -823,7 +850,8 @@ public:
 
             // immiscible gas
             const LhsEval Rv(0.0);
-            const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv);
+            const LhsEval Rvw(0.0);
+            const LhsEval& bg = gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
 
             return referenceDensity(phaseIdx, regionIdx)*bg;
 
@@ -875,6 +903,20 @@ public:
             return oilPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rs);
         }
         case gasPhaseIdx: {
+            if (enableVaporizedOil() && enableVaporizedWater()) {
+                 const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                 const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                 if (fluidState.saturation(waterPhaseIdx) > 0.0
+                    && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p))
+                    && fluidState.saturation(oilPhaseIdx) > 0.0
+                    && Rv >= (1.0 - 1e-10)*gasPvt_->saturatedOilVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
+                 { 
+                    return gasPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
+                 } else {
+                     return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+                 }
+            }
+
             if (enableVaporizedOil()) {
                 const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
                 if (fluidState.saturation(oilPhaseIdx) > 0.0
@@ -882,12 +924,26 @@ public:
                 {
                     return gasPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
                 } else {
-                    return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv);
+                    const LhsEval Rvw(0.0);
+                    return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
                 }
             }
 
+            if (enableVaporizedWater()) { 
+                const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                if (fluidState.saturation(waterPhaseIdx) > 0.0
+                    && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
+                {
+                    return gasPvt_->saturatedInverseFormationVolumeFactor(regionIdx, T, p);
+                } else {
+                    const LhsEval Rv(0.0);
+                    return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
+                }
+            }
+            
             const LhsEval Rv(0.0);
-            return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv);
+            const LhsEval Rvw(0.0);
+            return gasPvt_->inverseFormationVolumeFactor(regionIdx, T, p, Rv, Rvw);
         }
         case waterPhaseIdx:
             return waterPvt_->inverseFormationVolumeFactor(regionIdx, T, p, saltConcentration);
@@ -942,7 +998,7 @@ public:
         const LhsEval phi_oO = 20e3/p;
 
         // for the gas component in the gas phase, assume it to be an ideal gas
-        const Scalar phi_gG = 1.0;
+        constexpr const Scalar phi_gG = 1.0;
 
         // for the fugacity coefficient of the water component in the water phase, we use
         // the same approach as for the oil component in the oil phase
@@ -1075,6 +1131,19 @@ public:
         }
 
         case gasPhaseIdx: {
+             if (enableVaporizedOil() && enableVaporizedWater()) {
+                 const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                 const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                 if (fluidState.saturation(waterPhaseIdx) > 0.0
+                    && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p))
+                    && fluidState.saturation(oilPhaseIdx) > 0.0
+                    && Rv >= (1.0 - 1e-10)*gasPvt_->saturatedOilVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
+                 { 
+                     return gasPvt_->saturatedViscosity(regionIdx, T, p);
+                 } else {
+                     return gasPvt_->viscosity(regionIdx, T, p, Rv, Rvw);
+                 }
+            }
             if (enableVaporizedOil()) {
                 const auto& Rv = BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
                 if (fluidState.saturation(oilPhaseIdx) > 0.0
@@ -1082,12 +1151,25 @@ public:
                 {
                     return gasPvt_->saturatedViscosity(regionIdx, T, p);
                 } else {
-                    return gasPvt_->viscosity(regionIdx, T, p, Rv);
+                    const LhsEval Rvw(0.0);
+                    return gasPvt_->viscosity(regionIdx, T, p, Rv, Rvw);
+                }
+            }
+            if (enableVaporizedWater()) {
+                const auto& Rvw = BlackOil::template getRvw_<ThisType, FluidState, LhsEval>(fluidState, regionIdx);
+                if (fluidState.saturation(waterPhaseIdx) > 0.0
+                    && Rvw >= (1.0 - 1e-10)*gasPvt_->saturatedWaterVaporizationFactor(regionIdx, scalarValue(T), scalarValue(p)))
+                {
+                    return gasPvt_->saturatedViscosity(regionIdx, T, p); 
+                } else {
+                    const LhsEval Rv(0.0);
+                    return gasPvt_->viscosity(regionIdx, T, p, Rv, Rvw);
                 }
             }
 
             const LhsEval Rv(0.0);
-            return gasPvt_->viscosity(regionIdx, T, p, Rv);
+            const LhsEval Rvw(0.0);
+            return gasPvt_->viscosity(regionIdx, T, p, Rv, Rvw);
         }
 
         case waterPhaseIdx:
@@ -1119,12 +1201,12 @@ public:
 
         case gasPhaseIdx:
             return
-                gasPvt_->internalEnergy(regionIdx, T, p, BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx))
-                + p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
+                 gasPvt_->internalEnergy(regionIdx, T, p, BlackOil::template getRv_<ThisType, FluidState, LhsEval>(fluidState, regionIdx))
+                  + p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
 
         case waterPhaseIdx:
             return
-                waterPvt_->internalEnergy(regionIdx, T, p)
+                waterPvt_->internalEnergy(regionIdx, T, p, BlackOil::template getSaltConcentration_<ThisType, FluidState, LhsEval>(fluidState, regionIdx))
                 + p/density<FluidState, LhsEval>(fluidState, phaseIdx, regionIdx);
 
         default: throw std::logic_error("Unhandled phase index "+std::to_string(phaseIdx));
