@@ -29,7 +29,6 @@
 
 #include <opm/material/Constants.hpp>
 
-#include <opm/material/common/Tabulated1DFunction.hpp>
 #include <opm/material/components/Brine.hpp>
 #include <opm/material/components/SimpleHuDuanH2O.hpp>
 #include <opm/material/components/CO2.hpp>
@@ -56,8 +55,7 @@ namespace Opm {
 template <class Scalar>
 class BrineCo2Pvt
 {
-    typedef std::vector<std::pair<Scalar, Scalar> > SamplingPoints;
-    static const bool extrapolate = true;
+    static constexpr bool extrapolate = true;
     //typedef H2O<Scalar> H2O_IAPWS;
     //typedef Brine<Scalar, H2O_IAPWS> Brine_IAPWS;
     //typedef TabulatedComponent<Scalar, H2O_IAPWS> H2O_Tabulated;
@@ -68,14 +66,12 @@ class BrineCo2Pvt
 
 
 public:
-    typedef SimpleHuDuanH2O<Scalar> H2O;
-    typedef ::Opm::Brine<Scalar, H2O> Brine;
-    typedef ::Opm::CO2<Scalar, CO2Tables> CO2;
-
-    typedef Tabulated1DFunction<Scalar> TabulatedOneDFunction;
+    using H2O = SimpleHuDuanH2O<Scalar>;
+    using Brine = ::Opm::Brine<Scalar, H2O>;
+    using CO2 = ::Opm::CO2<Scalar, CO2Tables>;
 
     //! The binary coefficients for brine and CO2 used by this fluid system
-    typedef BinaryCoeff::Brine_CO2<Scalar, H2O, CO2> BinaryCoeffBrineCO2;
+    using BinaryCoeffBrineCO2 = BinaryCoeff::Brine_CO2<Scalar, H2O, CO2>;
 
     explicit BrineCo2Pvt() = default;
     BrineCo2Pvt(const std::vector<Scalar>& brineReferenceDensity,
@@ -96,11 +92,11 @@ public:
         int num_regions =  salinity_.size();
         co2ReferenceDensity_.resize(num_regions);
         brineReferenceDensity_.resize(num_regions);
+        Brine::salinity = salinity[0];
         for (int i = 0; i < num_regions; ++i) {
             co2ReferenceDensity_[i] = CO2::gasDensity(T_ref, P_ref, true);
             brineReferenceDensity_[i] = Brine::liquidDensity(T_ref, P_ref, true);
         }
-        Brine::salinity = salinity[0];
     }
 #if HAVE_ECL_INPUT
     /*!
@@ -119,6 +115,8 @@ public:
                          "BRINE PVT properties are computed based on the Hu et al. pvt model and PVDO/PVTO input is ignored. " << std::endl;
         }
 
+        setEnableDissolvedGas(eclState.getSimulationConfig().hasDISGAS());
+
         // We only supported single pvt region for the co2-brine module
         size_t numRegions = 1;
         setNumRegions(numRegions);
@@ -167,6 +165,15 @@ public:
 
     }
 
+    /*!
+     * \brief Specify whether the PVT model should consider that the CO2 component can
+     *        dissolve in the brine phase
+     *
+     * By default, dissolved co2 is considered.
+     */
+    void setEnableDissolvedGas(bool yesno)
+    { enableDissolution_ = yesno; }
+
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
@@ -313,6 +320,7 @@ private:
     std::vector<Scalar> brineReferenceDensity_;
     std::vector<Scalar> co2ReferenceDensity_;
     std::vector<Scalar> salinity_;
+    bool enableDissolution_ = true;
 
     template <class LhsEval>
     LhsEval density_(unsigned regionIdx,
@@ -442,6 +450,9 @@ private:
                    const LhsEval& temperature,
                    const LhsEval& pressure) const
     {
+        if (!enableDissolution_)
+            return 0.0;
+
         // calulate the equilibrium composition for the given
         // temperature and pressure. 
         LhsEval xgH2O;
@@ -471,12 +482,12 @@ private:
         /* same function as enthalpy_brine, only extended by CO2 content */
 
         /*Numerical coefficents from PALLISER*/
-        static Scalar f[] = {
+        static constexpr Scalar f[] = {
             2.63500E-1, 7.48368E-6, 1.44611E-6, -3.80860E-10
         };
 
         /*Numerical coefficents from MICHAELIDES for the enthalpy of brine*/
-        static Scalar a[4][3] = {
+        static constexpr Scalar a[4][3] = {
             { 9633.6, -4080.0, +286.49 },
             { +166.58, +68.577, -4.6856 },
             { -0.90963, -0.36524, +0.249667E-1 },
