@@ -29,7 +29,6 @@
 
 #include <opm/material/Constants.hpp>
 #include <opm/material/common/MathToolbox.hpp>
-#include <opm/material/common/OpmFinal.hpp>
 #include <opm/material/common/UniformXTabulated2DFunction.hpp>
 #include <opm/material/common/Tabulated1DFunction.hpp>
 
@@ -52,11 +51,11 @@ namespace Opm {
 template <class Scalar>
 class DryHumidGasPvt
 {
-    typedef std::vector<std::pair<Scalar, Scalar> > SamplingPoints;
+    using SamplingPoints = std::vector<std::pair<Scalar, Scalar>>;
 
 public:
-    typedef UniformXTabulated2DFunction<Scalar> TabulatedTwoDFunction;
-    typedef Tabulated1DFunction<Scalar> TabulatedOneDFunction;
+    using TabulatedTwoDFunction = UniformXTabulated2DFunction<Scalar>;
+    using TabulatedOneDFunction = Tabulated1DFunction<Scalar>;
 
     DryHumidGasPvt()
     {
@@ -324,13 +323,13 @@ public:
     {
         auto& waterVaporizationFac = saturatedWaterVaporizationFactorTable_[regionIdx];
 
-        Scalar RwMin = 0.0;
+        constexpr const Scalar RwMin = 0.0;
         Scalar RwMax = waterVaporizationFac.eval(saturatedWaterVaporizationFactorTable_[regionIdx].xMax(), /*extrapolate=*/true);
 
         Scalar poMin = samplePoints.front().first;
         Scalar poMax = samplePoints.back().first;
 
-        size_t nRw = 20;
+        constexpr const size_t nRw = 20;
         size_t nP = samplePoints.size()*2;
 
         TabulatedOneDFunction mugTable;
@@ -425,10 +424,11 @@ public:
     Evaluation viscosity(unsigned regionIdx,
                          const Evaluation& /*temperature*/,
                          const Evaluation& pressure,
-                         const Evaluation& Rw) const
+                         const Evaluation& /*Rv*/,
+                         const Evaluation& Rvw) const
     {
-        const Evaluation& invBg = inverseGasB_[regionIdx].eval(pressure, Rw, /*extrapolate=*/true);
-        const Evaluation& invMugBg = inverseGasBMu_[regionIdx].eval(pressure, Rw, /*extrapolate=*/true);
+        const Evaluation& invBg = inverseGasB_[regionIdx].eval(pressure, Rvw, /*extrapolate=*/true);
+        const Evaluation& invMugBg = inverseGasBMu_[regionIdx].eval(pressure, Rvw, /*extrapolate=*/true);
 
         return invBg/invMugBg;
     }
@@ -454,8 +454,9 @@ public:
     Evaluation inverseFormationVolumeFactor(unsigned regionIdx,
                                             const Evaluation& /*temperature*/,
                                             const Evaluation& pressure,
-                                            const Evaluation& Rw) const
-    { return inverseGasB_[regionIdx].eval(pressure, Rw, /*extrapolate=*/true); }
+                                            const Evaluation& /*Rv*/,
+                                            const Evaluation& Rvw) const
+    { return inverseGasB_[regionIdx].eval(pressure, Rvw, /*extrapolate=*/true); }
 
     /*!
      * \brief Returns the formation volume factor [-] of water saturated gas at a given pressure.
@@ -608,7 +609,7 @@ public:
     bool operator==(const DryHumidGasPvt<Scalar>& data) const
     {
         return this->gasReferenceDensity_ == data.gasReferenceDensity_ &&
-               this->oilReferenceDensity_ == data.waterReferenceDensity_ &&
+               this->waterReferenceDensity_ == data.waterReferenceDensity_ &&
                this->inverseGasB() == data.inverseGasB() &&
                this->inverseSaturatedGasB() == data.inverseSaturatedGasB() &&
                this->gasMu() == data.gasMu() &&
@@ -643,7 +644,8 @@ private:
         //Prune duplicate Rv values (can occur, and will cause problems in further interpolation)
         auto x_coord_comparator = [](const Pair& a, const Pair& b) { return a.first == b.first; };
         auto last = std::unique(pSatSamplePoints.begin(), pSatSamplePoints.end(), x_coord_comparator);
-        pSatSamplePoints.erase(last, pSatSamplePoints.end());
+        if (std::distance(pSatSamplePoints.begin(), last) > 1) // only remove them if there are more than two points
+            pSatSamplePoints.erase(last, pSatSamplePoints.end());
 
         saturationPressure_[regionIdx].setContainerOfTuples(pSatSamplePoints);
     }
